@@ -40,7 +40,7 @@
 #' @export
 #' @importFrom methods is
 #' @importFrom stats .getXlevels binomial dpois fitted gaussian glm.fit 
-#' @importFrom stats make.link model.matrix model.offset model.response 
+#' @importFrom stats make.link model.matrix model.offset model.response coef
 #' @importFrom stats optim pchisq pnorm predict printCoefmat rpois update
 #' @import gamlss
 #' @import splines
@@ -52,6 +52,7 @@ SARARgamlss <- function(formula, sigma.formula = ~1,
                         type = c("SAR", "SARAR", "SEM"),
                         weights = NULL) {
   mf <- stats::model.frame(formula, data=data)
+  data_name <- substitute(data) 
   if (type == "SAR") {
     W2 = 0 * W2
   }
@@ -69,7 +70,7 @@ SARARgamlss <- function(formula, sigma.formula = ~1,
   
   # Fit the initial GAMLSS model for mean (mu) and variance (sigma)
   m0 <- gamlss::gamlss(formula = formula, sigma.formula = sigma.formula, 
-                       data = data, family = NO())
+                       data = eval(data_name, parent.frame()), family = NO())
   
   Y <- matrix(m0$y, ncol = 1)
   
@@ -158,10 +159,34 @@ SARARgamlss <- function(formula, sigma.formula = ~1,
     spacov =var_cov_matrix 
   }
   # model.frame(m0)[,1] <- as.vector(model.frame(m1)[,1])
-  m0$call$data <- data 
-  out1 <- list(gamlss=m0, model = mf, data=data,
+  m0$call$data <- data_name
+  
+  X <- model.matrix(m0, what = "mu")
+  beta <- stats::coef(m0, "mu")
+  Omega <- diag(predict(m0, what = "sigma")^2)
+  rho <- p0[1]
+  lambda <- p0[2]
+  AA <- diag(n) - rho * W1
+  BB <- diag(n) - lambda * W2
+  varU <- solve(BB) %*% Omega %*% t(solve(BB))
+  
+  inv_cov <- solve(lambda^2 * W2 %*% varU %*% t(W2) +
+                     lambda *  Omega %*% t(solve(BB)) %*% t(W2) +
+                     lambda * W2 %*% solve(BB) %*% Omega +
+                     Omega)
+  y_true <- Y
+  y_signal <- (lambda* W2 %*% varU + solve(BB) %*% Omega) %*% inv_cov %*% 
+    (AA%*%y_true -  X %*% beta)
+  
+  y_trend <- rho * W1 %*% y_true + X %*% beta
+  y_blup <- y_trend + y_signal
+  residuals <- AA%*%y_true -  X %*% beta
+  y_noise <- y_true - y_blup
+  
+  out1 <- list(gamlss=m0, model = mf,
                spatial=list(spatial=spamu, sdspatial=spacov, type=type),
-               gamlssAY=m1)
+               gamlssAY=m1,  y_signal = y_signal, y_trend = y_trend,
+               y_blup = y_blup, residuals = residuals,y_noise = y_noise)
   class(out1) <- "SARARgamlss"
   return(out1)
 }
